% Program modelbuild for generating FOPDT ( First Order Plus Dead Time) Models 
% From Input Output data
% Copyright  Babu Joseph, Washington University 2/2000
% This program uses the polynomial impulse response function fitting program 
% developed by C.Y.Ying
% For full details of the method used see Ying and Joseph, IEC Research, 38, pp4712,1999
% We will take the impulse response generated by the program and create a FOPDT model
%
function modelbuild(action)
global uuuu tttt yyyy

% access user workspace from saved file
   % This local function gets all handles and current variables from GUI
[gobject,hfigure]=gcbo;
Handle_u=findobj(hfigure,'Tag','u');
Handle_y=findobj(hfigure,'Tag','y');
Handle_t=findobj(hfigure,'Tag','t');
Handle_Tau_init=findobj(hfigure,'Tag','Tau_init');
Handle_Delay_init=findobj(hfigure,'Tag','Delay_init');
Handle_SSE=findobj(hfigure,'Tag','SSE');
Handle_Rsquared=findobj(hfigure,'Tag','Rsquared');
Handle_uplot=findobj(hfigure,'Tag','uplot');
Handle_yplot=findobj(hfigure,'Tag','yplot');
Handle_tstart=findobj(hfigure,'Tag','tstart');
Handle_tend=findobj(hfigure,'Tag','tend');

% Get current entries in slots

string_u=get(Handle_u,'String');
string_y=get(Handle_y,'String');
string_t=get(Handle_t,'String');
Tau_init=str2num(get(Handle_Tau_init,'String'));
Delay_init=str2num(get(Handle_Delay_init,'String'));

Handle_Gain=findobj(hfigure,'Tag','Gain');
Handle_Tau=findobj(hfigure,'Tag','Tau');
Handle_Delay=findobj(hfigure,'Tag','Delay');

Gain=str2num(get(Handle_Gain,'String'));
Tau=str2num(get(Handle_Tau,'String'));
Delay=str2num(get(Handle_Delay,'String'));
Gain=str2num(get(Handle_Gain,'String'));
Handle_tstart=findobj(hfigure,'Tag','tstart');
Handle_tend=findobj(hfigure,'Tag','tend');
tstart=str2num(get(Handle_tstart,'String'));
tend=str2num(get(Handle_tend,'String'));
% End of local function
 
switch(action)
   
case 'getdata',
   [yyyy,uuuu,tttt]=getdatafunction;
   % store data in global variables
   % enter names into the GUI
    evalin('base', 'save temp');
;

   
case 'plotdata',
   % plot the user data on gui
   %
   plotdata;
case 'init_guess'
   % get initial guesses from user using interaction
   % enter into gui
   % give suggestions for making guesses
   
case 'fit'
   [Gain, tau,theta]=fit(yyyy,uuuu,tttt);
   % get all params and data and invoke ying program
   % plot results on gui
   % show params on gui
	
case 'validate'
   % get validation data
   % run fit 
   % show results
case 'quit'
   % end session
   reply=questdlg('Save Current Data?','Quit session','Yes','No','Cancel','No');
   if strcmp(reply,'Yes');
      default={'mymodel'};
      replystring=inputdlg('Enter save filename','Save Current Session',1,default);
            if isempty(replystring); close(gcf);return
        else
           print(gcf,'-dmfile',char(replystring));
           iodata=[yyyy uuuu tttt];
           save ((strcat(char(replystring),'.dat')), 'iodata', '-ascii');
      close(gcf);
   end
elseif strcmp(reply,'No');close(gcf);return;
   elseif strcmp(reply,'Cancel');return
   end;   
   
   
end
% ********************************************
function [yyyy,uuuu,tttt]=getdatafunction
% ********************************************
% obtain user data
   % choose between workspace, mat file and ascii file
   % return variable names to gui and to global workspace
   % check data for discrepancies and alert user
   % plot the data on gui before asking questions of user
   reply=questdlg('Where is your data stored?','Get Data', 'Workspace','.mat file','ascii file','Workspace');
   if strcmp(reply,'Workspace')
      % Select input vector
      evalin('base', 'save temp');
      load temp;
      variables=who;no_var=length(variables);
      
      [selection,ok]=listdlg('PromptString','Select  INPUT variable','SelectionMode','single',...
         'ListString',variables);
      
      if ok   uselection=selection ;      end;
            if ~ok return ;end;

       [selection,ok]=listdlg('PromptString','Select OUTPUT variable','SelectionMode','single',...
         'ListString',variables);
      if ok  yselection=selection  ;   end;
            if ~ok return ;end;

      
      [selection,ok]=listdlg('PromptString','Select TIME variable','SelectionMode','single',...
         'ListString',variables);
      if ok ;tselection=selection;end;
            if ~ok return ;end;

 [gobject,hfigure]=gcbo;
Handle_u=findobj(hfigure,'Tag','u');
Handle_y=findobj(hfigure,'Tag','y');
Handle_t=findobj(hfigure,'Tag','t');

	if ok 
   set(Handle_u,'String',variables(uselection)); 
	set(Handle_y,'String',variables(yselection)); 
   set(Handle_t,'String',variables(tselection)); 
   yyyy=eval(char(variables(yselection)));
   uuuu=eval(char(variables(uselection)));
   tttt=eval(char(variables(tselection)));
   data_ok=checkdata(yyyy,uuuu,tttt);if not(data_ok) return;end;
   Handle_uplot=findobj(hfigure,'Tag','uplot');axes(Handle_uplot);
   cla % reset;
   plot(tttt,uuuu);title('Input Data');set(Handle_uplot,'Tag','uplot');
   set(Handle_uplot,'Tag','uplot');
   Handle_yplot=findobj(hfigure,'Tag','yplot');axes(Handle_yplot);
   cla %reset; 
   plot(tttt,yyyy);title('Output Data');set(Handle_yplot,'Tag','yplot');
   set(Handle_yplot,'Tag','yplot');
   tends=num2str(max(tttt));tstarts=num2str(min(tttt));
   Handle_tend=findobj(hfigure,'Tag','tend');set(Handle_tend,'String',tends);
   Handle_tstart=findobj(hfigure,'Tag','tstart');set(Handle_tstart,'String',tstarts);

end;
     
      
elseif strcmp(reply,'.mat file')
   % load the mat file specified by user
   [matfile,datapath]=uigetfile('*.mat','Choose a .mat file');
   if matfile
      eval(['load(''' datapath matfile ''')']);   end
   
   variables=who;no_var=length(variables);
      
      [selection,ok]=listdlg('PromptString','Select INPUT variable','SelectionMode','single',...
         'ListString',variables);
      if ok uuuu=eval(char(variables(selection)));uselection=selection; end
      if ~ok return ;end;
       [selection,ok]=listdlg('PromptString','Select OUTPUT variable','SelectionMode','single',...
         'ListString',variables);
      if ok yyyy=eval(char(variables(selection)));   yselection=selection;   end
            if ~ok return ;end;

      [selection,ok]=listdlg('PromptString','Select TIME variable','SelectionMode','single',...
         'ListString',variables);
      if ok tttt=eval(char(variables(selection)));tselection=selection;end
            if ~ok return; end;

       [gobject,hfigure]=gcbo;
Handle_u=findobj(hfigure,'Tag','u');
Handle_y=findobj(hfigure,'Tag','y');
Handle_t=findobj(hfigure,'Tag','t');
		if ok set(Handle_u,'String',variables(uselection)); 
		set(Handle_y,'String',variables(yselection));
      set(Handle_t,'String',variables(tselection));
      yyyy=eval(char(variables(yselection)));
      uuuu=eval(char(variables(uselection)));
      tttt=eval(char(variables(tselection)));
      data_ok=checkdata(yyyy,uuuu,tttt);if not(data_ok) return;end;
         Handle_uplot=findobj(hfigure,'Tag','uplot');axes(Handle_uplot);
   plot(tttt,uuuu);title('Input Data');set(Handle_uplot,'Tag','uplot');
      Handle_yplot=findobj(hfigure,'Tag','yplot');axes(Handle_yplot);

plot(tttt,yyyy);title('Output Data');set(Handle_yplot,'Tag','yplot');
   tends=num2str(max(tttt));tstarts=num2str(min(tttt));
   Handle_tend=findobj(hfigure,'Tag','tend');set(Handle_tend,'String',tends);
   Handle_tstart=findobj(hfigure,'Tag','tstart');set(Handle_tstart,'String',tstarts);


   end;


      
elseif strcmp(reply,'ascii file')
   message={'The ascii file must contain three columns of data','In the order  time input output'};
   
   
   Hd_error=helpdlg(message,'For Your Informaton');pause(3);
      
   [filename,pathname]=uigetfile('*.dat',' Opening input-output data file ');
fname=filename;
 
%disp ( '       Please enter the data file name');
%disp(  '       You must include the path if the data file is not in the current directory');
%disp(  '       You can change the current directory by typing " cd newdirectory" when you get the matlab prompt');
%disp (' ');
%
% *******************************************************************************
% Check if filename is valid 
% *******************************************************************************
%
if fname==0; 
disp('   Could not open file');cdir=pwd;
disp (['    File not in current directory ' cdir]);
disp ('      Try entering file name with path');disp([]);
disp (message);
disp ( 'program terminated. Try again')
end;
%
% *******************************************************************************
% function for opening data file
% *******************************************************************************

[fid,message]=fopen ([pathname filename] );

z=fscanf(fid,'%f',inf);
fclose(fid);


yyyy=z(3:3:length(z));
uuuu=z(2:3:length(z));
tttt=z(1:3:length(z));
data_ok=checkdata(yyyy,uuuu,tttt);if not(data_ok) return; end;
	 [gobject,hfigure]=gcbo;

   Handle_uplot=findobj(hfigure,'Tag','uplot');axes(Handle_uplot);
   plot(tttt,uuuu);title('Input Data');set(Handle_uplot,'Tag','uplot');
      Handle_yplot=findobj(hfigure,'Tag','yplot');axes(Handle_yplot);

plot(tttt,yyyy);title('Output Data');set(Handle_yplot,'Tag','yplot');
   tends=num2str(max(tttt));tstarts=num2str(min(tttt));
   Handle_tend=findobj(hfigure,'Tag','tend');set(Handle_tend,'String',tends);
   Handle_tstart=findobj(hfigure,'Tag','tstart');set(Handle_tstart,'String',tstarts);



end
return  % end of getdata 
   
   
   % ********************************************
   % start of plotting function
   % ********************************************
   function plotdata
   global tttt uuuu yyyy
   	 [gobject,hfigure]=gcbo;

   Handle_uplot=findobj(hfigure,'Tag','uplot');axes(Handle_uplot);
   plot(tttt,uuuu);title('Input Data');set(Handle_uplot,'Tag','uplot');
      Handle_yplot=findobj(hfigure,'Tag','yplot');axes(Handle_yplot);

   plot(tttt,yyyy);title('Output Data');set(Handle_yplot,'Tag','yplot');
	
return %  end of plot function


    % ********************************************
   % Start of data checking section
   % ********************************************

function data_ok=checkdata(yd,ud,time)
data_ok=1;
dtime=diff(time);
dtimemax=max(dtime);
dtimemin=min(dtime);
ts=dtimemin;

   % This program checks if user data is okay
   %
   
   sy=size(yd);
if (sy(1)==1)
   yd=yd';
end
if ((sy(1)~=1)&(sy(2)~=1))
   message={'We have a problem with the data',...
         'The output variable is a matrix',...
         'It should be a vector',...
         'Try another data set'};
      
   Hd_error=errordlg(message,'Error Dialog');pause(3);dataok=0;
end


su=size(ud);
if (su(1)==1)
   ud=ud';
end
if ((su(1)~=1)&(su(2)~=1))
      message={'We have a problem with the data',...
         'The input variable is a matrix',...
         'It should be a vector',...
         'Try another data set'};
      
   Hd_error=errordlg(message,'Error Dialog');pause(3);data_ok=0;

end


st=size(time);
if (st(1)==1)
   time=time';
end
if ((sy(1)~=1)&(sy(2)~=1))
      message={'We have a problem with the data',...
         'The time variable is a matrix',...
         'It should be a vector',...
         'Try another data set'};
      
   Hd_error=errordlg(message,'Error Dialog');pause(3);data_ok=0;


end

sy=size(yd);
su=size(ud);
st=size(time);

if max(sy)~= max(su)
      message={'We have a problem with the data',...
         'Length of input variable is not equal to that of output'};
      
   Hd_error=errordlg(message,'Error Dialog');pause(3);

end

if max(sy)~= max(st)
         message={'We have a problem with the data',...
         'Length of time variable is not equal to that of output'};
      
   Hd_error=errordlg(message,'Error Dialog');pause(3);


end

%detect if input/output is transposed.
ncheck=min([15 length(yd)]);
if (((max(yd(1:ncheck))-min(yd(1:ncheck)))< 0.000001))
         message={'We have a problem with the data',...
               'Output is not changing',...
            'You may have exchanged input and output vectors'};
      
   Hd_error=errordlg(message,'Error Dialog');pause(3);


end

%detect if the input does not change at all
if (max(ud)-min(ud)) < 0.000001
            message={'We have a problem with the data',...
                  'Your input data is constant',...
                  'There must be at least one step change in input'};
      
   Hd_error=errordlg(message,'Error Dialog','replace');


end

% too long data length
if (length(yd)>6000)
            message={'We may have a problem with the data',...
                  'You have a lot of data',...
               'It may take a long time to fit model'};
      
   Hd_error=helpdlg(message,'Warning Dialog');pause(3);

end
      
% too short data length
if (length(yd)<1)
               message={'We may have a problem with the data',...
                  'Your data set is empty',...
               };
      
   Hd_error=errordlg(message,'Warning Dialog');pause(3);data_ok=0;

end
if (length(yd)<100)
               message={'We may have a problem with the data',...
                  'Your data is very short in length',...
               'Your fitted model will not be very accurate'};
      
   Hd_error=helpdlg(message,'Warning Dialog');pause(3);

end


if (length(yd)~=length(ud))|(length(yd)~=length(time))
            message={'We have a problem with the data',...
            'Your input and output vectors are not of same length'};
      
      Hd_error=errordlg(message,'Error Dialog');pause(3);

end
% check if user entered initial guesses
[gobject,hfigure]=gcbo;
Handle_Tau_init=findobj(hfigure,'Tag','Tau_init');
Handle_Delay_init=findobj(hfigure,'Tag','Delay_init');
% Get current entries in slots

Tau_init=str2num(get(Handle_Tau_init,'String'));
Delay_init=str2num(get(Handle_Delay_init,'String'));


if (Tau_init < ts) 
   message={'We have a problem with the data',...
         'You must enter nonzero initial guess for the time constant'};
   Hd_error=helpdlg(message,'Error Dialog');pause(3);
end

   return % end of check data function
   
   
   % ********************************************
	%polynomial model identification
   % ********************************************

function [K, tau,theta]=fit(yda,uda,timea)

% store data 
yd=yda;ud=uda;time=timea;

[gobject,hfigure]=gcbo;
Handle_Tau_init=findobj(hfigure,'Tag','Tau_init');
Handle_Delay_init=findobj(hfigure,'Tag','Delay_init');
Handle_tstart=findobj(hfigure,'Tag','tstart');
Handle_tend=findobj(hfigure,'Tag','tend');

tstart=str2num(get(Handle_tstart,'String'));
tend=str2num(get(Handle_tend,'String'));

dtime=diff(time);
dtimemax=max(dtime);
dtimemin=min(dtime);
ts=dtimemin;
tau=str2num(get(Handle_Tau_init,'String'));
delay=str2num(get(Handle_Delay_init,'String'));

% fit data
% [hm,hh,hl,K, theta, tau]=polyid(yd,ud,time)
% return

% check data

if tstart < min(time); tstart=min(time);end;
if tend > max(time); tend=max(time); end;
if tend <= tstart; tend=max(time); end;

if delay <= 0 ; delay=0; end;
if delay > tau; delay=tau; end;

if tau<= 0 ; tau=(tend-tstart)/4; end;
if tau > ( tend-tstart)/4 ; tau=(tend-tstart)/4; end;
tmax=4*tau/ts; tmin=delay/ts;
   
   % 
% extract data between specified times
%

tstartn=max(1,floor((tstart-min(time))/ts));
tendn=floor((tend-min(time))/ts);
time=time(tstartn:tendn);yd=yd(tstartn:tendn);ud=ud(tstartn:tendn);

time=time-time(1);
time2=0:time(length(time))/ts;

if (dtimemax>10*dtimemin)
               message={'We may have a problem with the data',...
                  'There are sampling gaps in your data ',...
                  'Some longer than 10*sample time',...
               ' PROCEED ANYWAY?'};
      
   reply=questdlg(message,'Warning Dialog','No','YES!','YES');
	if strcmp(reply,'No') return; end;
end
   

if (dtimemax-dtimemin) > 0.00001
      message={'We may have a problem with the data',...
          'Your data are not evenly spaced in time',...
          'Linear Interpolation will be used'};
      
   Hd_error=helpdlg(message,'Warning Dialog');pause(3);


   % disp(' Warning: time in I/O data is not evenly spaced!!! Interpolation is used.');
%   time2=1:time(length(time))/ts;
   yd=interp1(time,yd,time2*ts,'line');
   ud=interp1(time,ud,time2*ts,'line');
%   time=time2;

end

if length(time2) < length(yd)
   yd=yd(1:length(time2));
   ud=ud(1:length(time2));
end

if length(time2)>length(yd)
   time2=time2(1:length(yd));
end


time=time2;


%disp('press any key to continue...')
%pause;
% *******************************************************************************
% open data file
% *******************************************************************************

[fid,message]=fopen ('opt.dat');

options=zeros(1,25);
for i=19:25
options(i)=fscanf(fid,'%f',1);
line=fgets(fid);
end

for i=1:18
options(i)=fscanf(fid,'%f',1);
line=fgets(fid);
end
   
fclose(fid);
%options=options';
so=size(options);
if so(1)~=1
   options=options';
end
otheroptions=options(19:25);
options=options(1:18);
%options


%in=input('remove mean from the input-output data? Y/N  ','s');
%if in=='y' |in=='Y'
if otheroptions(1)==1
   meanu=mean(ud);
   meany=mean(yd);
   yd=yd-meany;
   ud=ud-meanu;
disp('mean removed');
disp(' ');
   
end

if otheroptions(2)==1
   ud=detrend(ud);
   yd=detrend(yd);
   disp('detrend finished');
disp(' ');

end

if otheroptions(3)==1
   ud=ud-ud(1);
   yd=yd-yd(1);
   disp('initial steady state value subtracted from data');
disp(' ');

end

umax=max(ud);
umin=min(ud);

if abs(umax-umin) < 0.001+0.001*mean(ud);
   disp('Warning: input may be wrong');
end

   

sizey=size(yd);
sizeu=size(ud);
sizet=size(time);

if sizey(1)==1
      yd=yd';
end
if sizeu(1)==1
      ud=ud';
end
if sizet(1)==1
      time=time';
end


disp('               Polynomial Model Identification Begins...');
Start_time=datestr(now)
disp(' ');
tmax=4*tau/ts;tmin=delay/ts;
if otheroptions(4)==0
   %disp('Please give your estimate of dead time and time constant');
   %
   % Remind user to enter initial guesses for time constant

    %tmin=input('The dead time is:  ');
   %tmax=input('The time constant is:  ');
   % tmax=tmax*4;
   % tmin=tmin/ts;
   % tmax=tmax/ts;
else
   
   disp(' ');
   disp('Automatically estimating the dead time and time constant...'); 
   disp('please wait ...')
   %pause(3);
   size(yd);
   size(ud);
   
   [hm,hh,hl]=polyid1(yd,ud,15,1,length(yd)-2);
   zyd=conv(ud,hm);
   zyd=zyd(1:length(ud));
   o1=size(zyd);
   if o1(1)==1
      zyd=zyd';
   end
   ud1=ud;
   o1=size(ud1);
   if o1(1)==1
      ud1=ud1';
   end
   [tmin,tmax]=timest(zyd,ud1,ts);
  
   disp('           Initial estimates used: ');
   disp(strcat('      dead time =   ',num2str(tmin*ts)));
   disp(strcat('      time constant =   ',num2str(tmax*ts/4)));
%disp('  ');
%   tmin
 %  tmax
 %disp('You may refine the dead time and time constant');
 %in=input('Do you want to do optimization? Y/N ','s');
 %if in =='y' |in=='Y'
%    disp('Please wait ...');
 %   [tmin, tmax]=est1(yd, ud,7,ts,tmin,tmax,options);
 %end
 


end
%  
% Start loop for estimating time constant and delay. Do two times
%
for itrial=1:1;
%   tmin
 %  tmax


disp(' ');
if ( tmax < 0 )
   message={'You must enter some initial guesses for time constant and delay',...
         'Before identificaton can proceed'};
   disp('');
   tmax=10*ts+tmin;
   disp('Error!:tmax set to 10*ts because user did not enter initial guesses');
   disp('');
      disp('');

   Hd_error=errordlg(message,'Error Dialog');
   end
if tmax>length(yd)-1 
   tmax=(length(yd)-1)/2;
         message={'We may have a problem with the initial guess',...
          'Time constant estimate is too large compared to data length',...
          'Time constant estimate reset to datalength/2'};
      
   Hd_error=helpdlg(message,'Warning Dialog');pause(3);

   %disp('Warning: time constant may not be correctly set');
end

if tmin < 0
   tmin=ts;
            message={'We may have a problem with the initial guess',...
          'Dead time estimate is too small compared to data length',...
          'The revised estimate reset to sample time'};
      
   Hd_error=helpdlg(message,'Warning Dialog');pause(3);

   % disp('Warning: dead time may not be correctly set');
end


%disp(strcat('dead time estimate reset to=   ',num2str(tmin*ts)));
%disp(strcat('time constant estimate  =   ',num2str(tmax*ts/4)));

% disp(' ');

if otheroptions(5)~=1
   % N=input('the order of polynomial N = ');
   % N=N-1;
   N=6;
else
   disp('    Automatically estimating the polynomial order...'); 
   disp('    please wait ...')
%   pause(3);

   N=estn(yd,ud,tmin,tmax);
end
disp(' ');

disp(strcat('   Polynomial order used is ...  ',' ',num2str(N+1)));      

%N=6;
%tmin=30;
%tmax=300;
if otheroptions(6)==1
   %idplot([yd ud]);
   %pause;
   %N
   %tmin
   %tmax
   size(yd);
   size(ud);
   %disp('main');
[hm1,hh1,hl1,ee1]=polyid11(yd',ud',N,tmin,tmax);
hm1=hm1(1:floor(tmax))/ts;
hh1=hh1(1:floor(tmax))/ts;
hl1=hl1(1:floor(tmax))/ts;
%ee
%ee1
%if ee1 <ee
   hm=hm1/ts;
   hh=hh1/ts;
   hl=hl1/ts;
 else
   [hm,hh,hl,ee]=polyid1(yd,ud,N,tmin,tmax);
   
hm=hm/ts;
hh=hh/ts;
hl=hl/ts;

end

%
 yha=conv(ud,hm*ts);
 yha=yha(1:length(ud));

% clc;

K=0;theta=0;tau=0;
 % disp('You may refine the dead time and time constant by optimization');
 % in=input('Do you want to do optimization? Y/N ','s');
 % if in =='y' |in=='Y'
 % disp('Please wait ...');
 %if tmax <= 0 
 
  %  end;
 % end


if otheroptions(7)==1
   % estimate optimum tmin and tmax
    %[tmin, tmax]=est1(yd, ud,7,ts,tmin,tmax,options);
    % disp ('optimum estimates of dead time  and time constant are as follows')
    % [tmin*ts  tmax*ts/4]
   yd1=conv(hm*ts,ud);
   %   sizeyd1=
   yd1=yd1(1:length(ud));
   [K, theta, tau,yp5]=id1(yd1,ud,time*ts,tmin);
   	 [gobject,hfigure]=gcbo;
       Handle_uplot=findobj(hfigure,'Tag','uplot');
       axes(Handle_uplot);
       time_plot=time*ts + tstart;
       cla;plot(time_plot,ud); set(Handle_uplot,'Tag','uplot');

     Handle_yplot=findobj(hfigure,'Tag','yplot');axes(Handle_yplot);
     cla;
     plot(time_plot,yd);
     %plot(ts*[1:length(yd)],yd);
 hold on;
 plot(time_plot,yha,'r');hold off;
 title('Comparison of polynomial model vs data');set(Handle_yplot,'Tag','yplot');
 %  disp(' End of calculation');
  %
  % Calculate Sum of Square of error etc.
  %
  
  SSE=sum((yd-yha).*(yd-yha));Rsquared=sum(yha.*yha)/sum(yd.*yd);
  Handle_SSE=findobj(hfigure,'Tag','SSE');
  Handle_Rsquared=findobj(hfigure,'Tag','Rsquared');
  set(Handle_SSE,'String',num2str(SSE));
  set(Handle_Rsquared,'String',num2str(Rsquared));

%   figure
% plot(time*ts,yd);
% hold on
% plot(time*ts,yp5,'r');
% title('FOPDT model output vs measurements')
end
%
% Post results on Figure boxes
%
Handle_Gain=findobj(hfigure,'Tag','Gain');
Handle_Tau=findobj(hfigure,'Tag','Tau');
Handle_Delay=findobj(hfigure,'Tag','Delay');
set(Handle_Gain,'String',num2str(K));
set(Handle_Tau,'String',num2str(tau));
set(Handle_Delay,'String',num2str(theta));
%
% End of loop for iterating on tmax and tmin
% 
tmax=tau; tmin =theta;
   tmax=tmax*4;
   tmin=tmin/ts;
   tmax=tmax/ts;

end;
% disp (' Done with identificaion');
%disp(strcat('gain K is: ',num2str(K)));
%disp(strcat('time constant tau is: ', num2str(tau)));
%disp(strcat('dead time theta is:  ', num2str(theta)));
%disp('the first order plus dead time model identified is:');
%disp(' ');
%disp(' ');

%disp(num2str(K));
%disp(strcat('----------*exp(-',num2str(theta),'*s)'));
%disp(strcat('1+',num2str(tau),'*s'));
%disp(' ');
%disp('********************************************************');
%disp(' ');

disp(' ');

disp('********************************************************');
disp(' ');
disp('Thank you for using polynomial model identification! ');
time_end=datestr(now)
%disp('For more information, please contact')
%disp(' ');
%disp('Dr. Babu Joseph');
%disp('Department of Chemical Engineering');
%disp('Washington University');
%disp('St. Louis, MO 63130');
%disp('Email: joseph@wuche.wustl.edu');
%disp('  ');

   
   
   


   
   
