%polyid
%copyright: Chao-Ming Ying and Babu Joseph, Washington University, St. Louis, 1998
%polynomial model identification 
%input parameters: yd--system output; ud--system input; time--time vector
%output results: hm--identified impulse response; hh--95% confidence limit (high)
%                hl--95% confidence limit (low);K, theta, tau -- FOPDT results
%Usage: [hm,hh,hl,K, theta, tau]=polyid(yd,ud,time)
function [hm,hh,hl,K, theta, tau]=polyid(yd,ud,time)

close;
close;
clc; % clear everything
%
% *******************************************************************************
fig1=figure; %    Opening screen
% *******************************************************************************
%
h=text(.1,.7,'Polynomial Model Identification');
set(h, 'FontSize',20);

h1=text( .2,.4, ' Program to do identification with polynomial model');
set (h1, 'Color',[ 1 0 0]);

h1=text (.2,.3, ' Written by Chao-Ming Ying and Babu Joseph');
set (h1, 'Color',[ 1 0 0]);
h1=text(.2, .2 ,' Washington University, St.Louis, MO  1998'); axis('off');
set (h1, 'Color',[  1 0 0]);

set (h, 'Color',[ 1 0 0]);
pause (2)
%
% *******************************************************************************
% End of opening screen logo
% *******************************************************************************
%
close;close;
disp(' ');
disp(' ');
disp('*******************************************************************************');
disp('*                                                                             *');
%
% *******************************************************************************
% Ask user for file name
% *******************************************************************************
%
disp('*      This is a Matlab program to do identification with polynomial model    *'  );
disp('*      Programmed by Chao-Ming Ying and Babu Joseph,                          *'); 
disp('*      Washington University in St. Louis, MO 63130, April 1998               *');
disp('*                                                                             *');
disp('*******************************************************************************');
disp(' ');
disp('Fast sampling is required for polynomial model to work.');
disp('mostly time to steady state should be larger than 100 sample time')
%
% *******************************************************************************
% Get user's data file
% *******************************************************************************
%
%in=input('Are your data from file? Y/N  ','s');
disp('press any key to continue...');
pause;
if isempty(yd)
   disp('no input-output data supplied, read from file');
   %if in =='y' |in=='Y'
% disp('data must be in y u t format')  
[filename,pathname]=uigetfile('*.dat',' Opening input-output data file ');
fname=filename;
 
%disp ( '       Please enter the data file name');
%disp(  '       You must include the path if the data file is not in the current directory');
%disp(  '       You can change the current directory by typing " cd newdirectory" when you get the matlab prompt');
%disp (' ');
%
% *******************************************************************************
% Check if filename is valid 
% *******************************************************************************
%
if fname==0; 
disp('   Could not open file');cdir=pwd;
disp (['    File not in current directory ' cdir]);
disp ('      Try entering file name with path');disp([]);
disp (message);
disp ( 'program terminated. Try again')
end;
%
% *******************************************************************************
% open data file
% *******************************************************************************

[fid,message]=fopen ([pathname filename] );

z=fscanf(fid,'%f',inf);
fclose(fid);


yd=z(1:3:length(z));
ud=z(2:3:length(z));
time=z(3:3:length(z));



else
   

end

sy=size(yd);
su=size(ud);
st=size(time);

if max(sy)~= max(su)
   error('input length not equal to output length!');
end

if max(sy)~= max(st)
   error('time length not equal to output length!');
end
disp('press any key to continue...')
pause;
   disp('read option file which is used in optimization...');
   %if in =='y' |in=='Y'
%   pause(2);
   % disp('data must be in y u t format')  
[filename,pathname]=uigetfile('*.dat',' Opening option data file for optimization parameters');
fname=filename;
 
%disp ( '       Please enter the data file name');
%disp(  '       You must include the path if the data file is not in the current directory');
%disp(  '       You can change the current directory by typing " cd newdirectory" when you get the matlab prompt');
%disp (' ');
%
% *******************************************************************************
% Check if filename is valid 
% *******************************************************************************
%
if fname==0; 
disp('   Could not open file');cdir=pwd;
disp (['    File not in current directory ' cdir]);
disp ('      Try entering file name with path');disp([]);
disp (message);
disp ( 'program terminated. Try again')
end;
%
% *******************************************************************************
% open data file
% *******************************************************************************

[fid,message]=fopen ([pathname filename] );

options=fscanf(fid,'%f',18);
fclose(fid);
options=options';
%options

if (length(yd)~=length(ud))|(length(yd)~=length(time))
   disp('Your input and output data length must be the same');
end

dtime=diff(time);

%get max and min time step
dtimemax=max(dtime);
dtimemin=min(dtime);
ts=dtimemin;
time=time-time(1);
time2=0:time(length(time))/ts;


if (dtimemax-dtimemin) > 0.00001
   disp(' Warning: time in I/O data is not evenly spaced!!! Interpolation is used.');
%   time2=1:time(length(time))/ts;
   yd=interp1(time,yd,time2*ts,'line');
   ud=interp1(time,ud,time2*ts,'line');
%   time=time2;

end

if length(time2) < length(yd)
   yd=yd(1:length(time2));
   ud=ud(1:length(time2));
end

if length(time2)>length(yd)
   time2=time2(1:length(yd));
end


time=time2;

in=input('remove mean from the input-output data? Y/N  ','s');
if in=='y' |in=='Y'
   meanu=mean(ud);
   meany=mean(yd);
   yd=yd-meany;
   ud=ud-meanu;
disp('mean removed');
disp(' ');
   
end

disp('detrend will remove the linear trend in the data.');
in=input('detrend the input-output data? Y/N ','s');
if in=='y' | in=='Y'
   ud=detrend(ud);
   yd=detrend(yd);
   disp('detrend finished');
disp(' ');

end

in=input('subtract the initial steady state value u(1) and y(1) from IO data? Y/N ','s');
if in=='y' | in=='Y'
   ud=ud-ud(1);
   yd=yd-yd(1);
   disp('steady state value subtracted');
disp(' ');

end

umax=max(ud);
umin=min(ud);

if abs(umax-umin) < 0.001+0.001*mean(ud);
   disp('Warning: input may be wrong');
end

   
   subplot(211);


plot(ts*time,yd);
title('This is your data');
xlabel('time');
ylabel('output');

subplot(212);
plot(ts*time,ud);
title('input');
xlabel('time');
ylabel('input');
disp('Press any key to continue...');
disp(' ');

sizey=size(yd);
sizeu=size(ud);
sizet=size(time);
%disp('stop here')
%pause

if sizey(1)==1
      yd=yd';
end
if sizeu(1)==1
      ud=ud';
end
if sizet(1)==1
      time=time';
end
disp('press any key to continue...')

pause;
repeat='y';
repeatn=0;
while repeat=='y' | repeat=='Y'

disp('Polynomial Model Identification Begins...');
disp(' ');
disp('Do you want to specify the dead time ');
%junk=input('','s');
in=input('and time to steady state by yourself Y/N?  ','s');
if (in=='y') | (in=='Y')
   tmin=input('The dead time is:  ');
   tmax=input('The time to steady state is:  ');
   disp(' ');
   tmin=tmin/ts;
   tmax=tmax/ts;
else
   disp(' ');
   disp('Automatically estimating the dead time and time to steady state...'); 
   disp('please wait ...')
   %pause(1);
   size(yd);
   size(ud);
   
   %use polynomial to get an initial FIR
   [hm,hh,hl]=polyid1(yd,ud,15,1,length(yd)-2);
   zyd=conv(ud,hm);
   zyd=zyd(1:length(ud));
   o1=size(zyd);
   if o1(1)==1
      zyd=zyd';
   end
   ud1=ud;
   o1=size(ud1);
   if o1(1)==1
      ud1=ud1';
   end
   %use arx to get a dead time and final time estimate
   [tmin,tmax]=timest(zyd,ud1,ts);
  
   disp('Initial estimates: ');
   disp(strcat('dead time =   ',num2str(tmin*ts)));
disp(strcat('time to steady state =   ',num2str(tmax*ts)));
disp('  ');
%   tmin
 %  tmax
 disp('You may refine the dead time and time to steady state by optimization');
 in=input('Do you want to do optimization? Y/N ','s');
 if in =='y' |in=='Y'
    disp('Please wait ...');
    [tmin, tmax]=est1(yd, ud,7,ts,tmin,tmax,options); %optimization
 end
 


end
disp(' ');
if tmax>length(yd)-1 | tmax<1
   tmax=length(yd)-1;
   disp('Warning: time to steady state may not be correctly set');
end

if tmin<0 | tmin > tmax
   tmin=tmax/4;
   disp('Warning: dead time may not be correctly set');
end


disp(strcat('dead time =   ',num2str(tmin*ts)));
disp(strcat('time to steady state =   ',num2str(tmax*ts)));

disp(' ');

in=input('Do you want to specify the order of polynomial Y/N?  ','s');
if (in=='y')|(in=='Y')
   N=input('the order of polynomial N = ');
   N=N-1;
else
   disp('Automatically estimating the polynomial order...'); 
   disp('please wait ...')
%   pause(1);

   N=estn(yd,ud,tmin,tmax); %order estimate
end
disp(' ');

disp(strcat('polynomial order used is   ',' ',num2str(N+1)));      
%N=6;
%tmin=30;
%tmax=300;
in=input('First order system? Y/N   ','s');
if in=='y' | in=='Y'
   %idplot([yd ud]);
   %pause;
   %N
   %tmin
   %tmax
   size(yd);
   size(ud);
   %disp('main');
[hm1,hh1,hl1,ee1]=polyid11(yd',ud',N,tmin,tmax);
hm1=hm1(1:floor(tmax))/ts;
hh1=hh1(1:floor(tmax))/ts;
hl1=hl1(1:floor(tmax))/ts;
%ee
%ee1
%if ee1 <ee
   hm=hm1;
   hh=hh1;
   hl=hl1;
else
   
[hm,hh,hl,ee]=polyid1(yd,ud,N,tmin,tmax);
hm=hm/ts;
hh=hh/ts;
hl=hl/ts;

end

if repeatn==0
   figure;
end
repeatn=repeatn+1;
figure(2);
clf
plot(ts*[1:length(hm)],hm);
hold on
plot(ts*[1:length(hh)],hh,'r--');
plot(ts*[1:length(hl)],hl,'r--');
hold off
title('your system impulse response and 95% confidence limit');
disp(' ');
%disp('the impulse response is saved as variable hm in the workspace');
%disp('the 95% confidence limit is saved as variables hh, hl in the workspace');
disp(' ');
pause(3);

yha=conv(ud,hm*ts);
yha=yha(1:length(ud));
if repeatn==0
   figure;
end

figure(3)
clf
plot(ts*[1:length(yd)],yd);
hold on
plot(ts*[1:length(yd)],yha,'r');
title('Polynomial model output vs measurements');
disp('you may revise dead time, time to steady state and polynomial order');
in = input('want to revise? Y/N ','s');
repeat='n';
if in =='y' |in =='Y'
   repeat='y';
   clc;
end
end
   


clc;
K=0;
theta=0;
tau=0;

disp('Do you want to get a FOPDT model from the result');
in=input('of polynomial identification model? Y/N ','s');
if in=='y' | in=='Y'
   yd1=conv(hm*ts,ud);
   %   sizeyd1=
   yd1=yd1(1:length(ud));
   [K, theta, tau,yp5]=id1(yd1,ud,time*ts,tmin);
   figure
plot(time*ts,yd);
hold on
plot(time*ts,yp5,'r');
title('FOPDT model output vs measurements')
end


disp(' ');
disp('Thank you for using polynomial model identification! ');
disp('For more information, please contact')
disp(' ');
disp('Dr. Babu Joseph');
disp('Department of Chemical Engineering');
disp('Washington University');
disp('St. Louis, MO 63130');
disp('Email: joseph@wuche.wustl.edu');
disp('  ');
